/*!
 * @file main.c
 * @brief Temp ISO Click Example.
 *
 * # Description
 * This example demonstrates the use of Temp ISO click board by reading
 * and displaying the temperature measurements.
 *
 * The demo application is composed of two sections :
 *
 * ## Application Init
 * Initializes the driver and logger.
 *
 * ## Application Task
 * Reads the temperature measurement in degrees Celsius and displays
 * the results on the USB UART approximately once per second.
 *
 * @author Stefan Filipovic
 *
 */

#include "board.h"
#include "log.h"
#include "tempiso.h"

static tempiso_t tempiso;   /**< Temp ISO Click driver object. */
static log_t logger;    /**< Logger object. */

void application_init ( void )
{
    log_cfg_t log_cfg;  /**< Logger config object. */
    tempiso_cfg_t tempiso_cfg;  /**< Click config object. */

    /** 
     * Logger initialization.
     * Default baud rate: 115200
     * Default log level: LOG_LEVEL_DEBUG
     * @note If USB_UART_RX and USB_UART_TX 
     * are defined as HAL_PIN_NC, you will 
     * need to define them manually for log to work. 
     * See @b LOG_MAP_USB_UART macro definition for detailed explanation.
     */
    LOG_MAP_USB_UART( log_cfg );
    log_init( &logger, &log_cfg );
    log_info( &logger, " Application Init " );

    // Click initialization.
    tempiso_cfg_setup( &tempiso_cfg );
    TEMPISO_MAP_MIKROBUS( tempiso_cfg, MIKROBUS_1 );
    if ( ADC_ERROR == tempiso_init( &tempiso, &tempiso_cfg ) )
    {
        log_error( &logger, " Communication init." );
        for ( ; ; );
    }
    
    log_info( &logger, " Application Task " );
}

void application_task ( void ) 
{
    float temperature = 0;
    if ( TEMPISO_OK == tempiso_read_temperature ( &tempiso, &temperature ) ) 
    {
        log_printf( &logger, " Temperature: %.1f degC\r\n\n", temperature );
        Delay_ms ( 1000 );
    }
}

int main ( void ) 
{
    /* Do not remove this line or clock might not be set correctly. */
    #ifdef PREINIT_SUPPORTED
    preinit();
    #endif
    
    application_init( );
    
    for ( ; ; ) 
    {
        application_task( );
    }

    return 0;
}

// ------------------------------------------------------------------------ END
